# Generated by Django 2.2.5 on 2019-09-20 05:31

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="CamundaConfig",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "root_url",
                    models.URLField(
                        default="http://localhost:12018/gemeente-utrecht/camunda/",
                        help_text="Root URL where camunda is installed. The REST api path is appended to this.",
                        verbose_name="camunda root",
                    ),
                ),
                (
                    "rest_api_path",
                    models.CharField(
                        default="engine-rest",
                        max_length=255,
                        verbose_name="REST api path",
                    ),
                ),
            ],
            options={"verbose_name": "Camunda configuration"},
        )
    ]
