# Generated by Django 3.0.3 on 2020-02-27 15:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("django_camunda", "0002_camundaconfig_auth_header")]

    operations = [
        migrations.AlterField(
            model_name="camundaconfig",
            name="root_url",
            field=models.URLField(
                default="https://camunda.example.com",
                help_text="Root URL where camunda is installed. The REST api path is appended to this.",
                verbose_name="camunda root",
            ),
        )
    ]
